/*
   Цвето-цветомузыка на Arduino и адресной светодиодной ленте WS2812b
 
   *******************************************************************
   Разработано: AlexGyver
   Страница проекта: http://alexgyver.ru/colormusic/
   GitHub: https://github.com/AlexGyver/ColorMusic

   Доработано: OlegAnadyr
               technotrasher

   Переработано: Slenk

   Отдельная благодарность: Aleksandr1612
   *******************************************************************

   Версия 2.4: 
   
   Данная модификация поддерживает 300 светодиодов.     
   Использует библиотеки: FastLED      - управление лентой
                          IRLremote    - ИК пульт
                          FHT          - преобразование Хартли
                          EEPROMex     - доступ к энергонезависимой памяти
                          GyverButton  - управление кнопками                             
   Библиотеки идут с проектом: https://community.alexgyver.ru/threads/cvetomuzyka-na-arduino.37/post-39347

   Особенности:
    - Плавная анимация (можно настроить)
    - Автонастройка по громкости (можно настроить)
    - Фильтр нижнего шума (можно настроить)
    - Автокалибровка шума при запуске (можно настроить)
    - В режиме частот лента не гаснет полностью (EMPTY_BRIGHT)
    - Все настройки сохраняются в памяти и не сбрасываются при перезагрузке
    - Сохранение настроек происходит при переводе в режим ожидания кнопкой ноль (0)
    - А также через 30 секунд после последнего нажатия на любую кнопку ИК пульта

   Автоматическая настройка нижнего порога шума для режимов цветомузыки ( 4-9 ) раздельно для 4-7 и 8-9
     1 - Включаем систему.
     2 - Выключаем музыку.
     3 - Жмём кнопку OK 4 раза подряд на пульте.
     4 - Значения шумов будут записаны в память и сами загружаться при последующем запуске.

   Ручная настройка нижнего порога шума для режимов цветомузыки ( 4-9 ) раздельно для 4-8 и 9
     1 - Включаем систему.
     2 - Включаем музыку.
     3 - Жмём кнопку OK (включается режим настроек - первые 8 диодов постоянно горят "радугой").
     4 - Кнопаками * / # настраиваем чувствиительность.
     5 - Жмём кнопку OK (выключается режим настроек).

   Сброс настроек в памяти
     1 - Выключаем систему кнопкой ноль (0) - первый диод тускло мигает красным.
     2 - Жмём кнопку OK 4 раза подряд на пульте.
     3 - Система перезагрузится со стандартными настройками.

    Настройка своего пульта:
     1 - Изменить параметр "REMOTE_LOG" с "0" на "1".
     2 - Загрузить скетч.
     3 - Открыть монито. 
     4 - Нажимаеть кнопки на пульте и записывать их в скетч в нужное место.

   Пульт:
  |------------------------------------------------------------------------------|
  | №  | Режим                         |       Кнопки ⇆     |     Кнопки ⇅     |
  |------------------------------------------------------------------------------|
  | 0   | Режим ожидания                | -                  | -                 |
  |------------------------------------------------------------------------------|
  | 1   | Подсветка одним цветом        |     Кнопки * / # - Смена подрежима     |
  | 1.1 | Белый свет                    | Температура        | Яркость           |
  | 1.2 | Постоянный цвет               | Цвет               | Насыщенность      |
  | 1.3 | Плавная смета цвета           | Скорость           | Насыщенность      |
  | 1.4 | Пульсация случайным цветом    | Скорость           | Насыщенность      |
  | 1.5 | Резкая смена цвета            | Скорость           | Насыщенность      |
  |------------------------------------------------------------------------------|
  | 2   | Цветовые эффекты              |     Кнопки * / # - Смена подрежима     |
  | 2.1 | Плавная радуга                | Скорость           | Шаг радуги        |
  | 2.2 | Плавная радуга с искрами      | Скорость           | Шаг радуги        |  
  | 2.3 | Случайная вспышка и затухание | Скорость           | Насыщенность      |
  | 2.4 | Случайная смена цветов        | Скорость           | Насыщенность      |
  | 2.5 | Безумие случайных вспышек     | Скорость           | Насыщенность      |
  | 2.6 | Безумие случайных вспышек 2   | Скорость           | Насыщенность      |
  | 2.7 | Конфетти                      | Скорость           | Насыщенность      |
  | 2.8 | Бегающая точка со следами     | Скорость           | Насыщенность      |
  | 2.9 | Пульсирующие цветные полосы   | Скорость           | -                 |
  | 2.10| 8 сплетающихся цветных точек  | -                  | Насыщенность      |  
  | 2.11| Огонь в центр                 | Скорость           | Цвет              |
  | 2.12| Лёд и Пламень                 | Скорость           | Цвет              |
  | 2.13| Огонь                         | Скорость           | Цвет              |
  | 2.14| Лава                          | Скорость           | Цвет              |
  | 2.15| Облака                        | Скорость           | Цвет              |
  | 2.16| Бассейн                       | Скорость           | Цвет              |
  | 2.17| Лес                           | Скорость           | Цвет              |
  | 2.18| Плазма                        | Скорость           | Цвет              |
  | 2.19| Зебра                         | Скорость           | Цвет              |
  |------------------------------------------------------------------------------|
  | 6   | Стробоскоп                    |       Кнопки * / # - Смена цвета       |  
  |  повторное нажатие - белый/цветной  | Плавность вспышек  | Частота вспышек   |
  |------------------------------------------------------------------------------|
  | 4   | Цветомузыка (1 полоса)        | Плавность анимации | Чувствительность  |
  |------------------------------------------------------------------------------|
  | 5   | Бегущие частоты               |     Кнопки * / # - Смена подрежима     |
  | 5.1 | 3 частоты                     | Скорость           | Чувствительность  |
  | 5.2 | Низкие                        | Скорость           | Чувствительность  |
  | 5.3 | Средние                       | Скорость           | Чувствительность  |
  | 5.4 | Высокие                       | Скорость           | Чувствительность  |
  |------------------------------------------------------------------------------|
  | 6   | Цветомузыка (3/5 полос)       |     Кнопки * / # - Смена подрежима     |
  | 6.1 | 3 полосы                      | Плавность анимации | Чувствительность  |
  | 6.2 | 5 полос                       | Плавность анимации | Чувствительность  |
  |------------------------------------------------------------------------------|
  | 7   | Анализатор спектра            | Шаг цвета          | Цвет              |
  |------------------------------------------------------------------------------|
  | 7   | Огоньки наперегонки           |     Кнопки * / # - Смена подрежима     |
  | 7.1 | С краю                ммм     | Скорость           | Насыщенность      |
  | 7.2 | Из центра                     | Скорость           | Насыщенность      |
  |------------------------------------------------------------------------------|        
  | 9   | Шкала громкости               |     Кнопки * / # - Смена подрежима     |
  | 9.1 | Градиент                      | Плавность анимации | -                 |
  | 9.2 | Радуга                        | Плавность анимации | Скорость радуги   |
  | 9.3 | Огонь                         | Плавность анимации | Цвет              |
  |------------------------------------------------------------------------------|
  |    Кнопка «ОК» - режим настроек     | Яркость горящих    | Яркость не горящих|
  |     (первые 8 диодов - радуга)      | Кнопки * и # - ручная настройка «шума» |
  |------------------------------------------------------------------------------|
  |     В режиме «Белый свет» (1.1)     |         Сбрасывает температуру         |
  |------------------------------------------------------------------------------|
  |      Кнопка «ОК» 4 раза подряд      |                                        |
  |     в режимах цветомузыки (4-9)     |   Авто настройка нижнего порога шума   |
  |       в выключенном состоянии       |        Сброс настроек в памяти         |
  |------------------------------------------------------------------------------|
*/
